
From: Andy Whitcroft <apw@shadowen.org>

Recent patches introduced a top down user process address space allocation
policy; further patches enable this for ppc64.  Although these work
correctly for normal maps, the topdown algorithm does not take into account
stringent mixing constraints for small and large pages on this
architecture.  These patches introduce a ppc64 specific
arch_get_unused_area_topdown() variant.  The first introduces
infrastructure to allow replacement of the generic
arch_get_unused_area_topdown() and the second utilises this infrastructure.

In this patch I have followed the pattern set by the arch_get_unused_area()
using HAVE_ARCH_UNMAPPED_AREA_TOPDOWN to be consistent.  However, it would
also be possible to simply have a ppc64_get_unused_area_topdown() in the
arch/ppc64/mm/mmap.c or to use weak bindings.

This patch:

Allow an architecture to override the default definition of
arch_get_unmapped_area_topdown().

Signed-off-by: Andy Whitcroft <apw@shadowen.org>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/mm/mmap.c |    2 ++
 1 files changed, 2 insertions(+)

diff -puN mm/mmap.c~ppc64-topdown-support mm/mmap.c
--- 25/mm/mmap.c~ppc64-topdown-support	Wed Sep  1 15:06:40 2004
+++ 25-akpm/mm/mmap.c	Wed Sep  1 15:06:40 2004
@@ -1106,6 +1106,7 @@ void arch_unmap_area(struct vm_area_stru
  * This mmap-allocator allocates new areas top-down from below the
  * stack's low limit (the base):
  */
+#ifndef HAVE_ARCH_UNMAPPED_AREA_TOPDOWN
 unsigned long
 arch_get_unmapped_area_topdown(struct file *filp, const unsigned long addr0,
 			  const unsigned long len, const unsigned long pgoff,
@@ -1190,6 +1191,7 @@ fail:
 
 	return addr;
 }
+#endif
 
 void arch_unmap_area_topdown(struct vm_area_struct *area)
 {
_
